/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
var app = {
    // Application Constructor
    initialize: function() {
        this.model = new PitcherModel();
        this.templates = {};
        this.curPage;        
        this.pitcherview = new PitcherView();
        this.menuview = new MenuView();
        this.aboutview = new AboutView();
        this.helpview = new HelpView();
        this.profileview = new ProfileView();
        this.alertview = new AlertView();
        this.selectcountview = new SelectCountView();
        this.inviteview = new InviteView();
        this.detailview = new DetailView();
        this.thnxview = new ThnxView();
        this.shareview = new ShareView();
        this.burolistview = new BurolistView();

        this.bindEvents();

    },
    closeInactivePages: function() {
        if (app.curPage != 'share') app.shareview.hide();
        if (app.curPage != 'help') app.helpview.hide();
        if (app.curPage != 'thnx') app.thnxview.hide();
        if (app.curPage != 'about') app.aboutview.hide();
        if (app.curPage != 'invite') app.inviteview.hide();
        if (app.curPage != 'profile' && app.curPage != 'pitcher' && app.curPage != 'burolist' && app.curPage != 'invite') app.profileview.hide();
        if (app.curPage == 'pitcher') app.detailview.hide();
        if (app.curPage == 'profile') {
            //app.profileview.hide();
            app.pitcherview.hide();
            app.detailview.hide();
            app.burolistview.hide();
            app.inviteview.hide();
            app.thnxview.hide();
        }
    },
    closeDetail: function  ( choice) {
   

        app.detailview.hide();

        switch( choice ) {

            case 'none':
            break;
            case 'select':
                setTimeout(function  () {
                   app.pitcherview.selectAgency( app.pitcherview );
                }, 300);
            break;
            case 'delete':
                setTimeout(function  () {
                    app.pitcherview.deleteAgency( app.pitcherview );
                }, 300);
            break;

        }
         
    },
    // Bind Event Listeners
    //
    // Bind any events that are required on startup.
    bindEvents: function() {
        document.addEventListener('deviceready', this.onDeviceEvent, false);
        document.addEventListener('menubutton', this.onDeviceEvent, false);

    },
    gotoPage: function(page) {


        // store page
        if (page != 'menu' && page != 'closemenu' && page != 'share' && page != 'detail') {
            app.curPage = page;
            app.closeInactivePages();

        }
        switch (page) {
            case 'menu':
                $("#menuview").removeClass("menu-close");
                 $("#menuoverlay").removeClass("overlay-close");
                break;
            case 'closemenu':
                $("#menuview").addClass("menu-close");
                 $("#menuoverlay").addClass("overlay-close");
                break;
            case 'about':                
                app.aboutview.show();
                break;
            case 'help':
                app.helpview.show();
                break;
            case 'profile':

                
                app.model.resetSearch();
                app.profileview.show();

                break;
            case 'pitcher':
                            
                app.pitcherview.show();

                app.model.firstTimeMsg = window.localStorage.getItem("firstTimeMsg");
        
                if( !app.model.firstTimeMsg ) {
                    app.alertview.show('firsttime');
                    window.localStorage.setItem("firstTimeMsg", true);
                }

               
                break;
            case 'detail':                
                app.detailview.show();
                break;
            case 'burolist':
                app.burolistview.show();
                break;
            case 'invite':
                
                app.inviteview.show();
                break;
            case 'thnx':
                app.thnxview.show();
                app.model.searchStarted = false;
                break;
        }
    },
    // Update DOM on a Received Event
    onDeviceEvent: function(e) {
       
        switch (e.type) {
            case 'deviceready':

                navigator.splashscreen.show(); // for android / ios in config.xml

               StatusBar.hide();
           

                // register handelbars helpers
                Handlebars.registerHelper('compare', function (lvalue, operator, rvalue, options) {

                    var operators, result;
                    
                    if (arguments.length < 3) {
                        throw new Error("Handlerbars Helper 'compare' needs 2 parameters");
                    }
                    
                    if (options === undefined) {
                        options = rvalue;
                        rvalue = operator;
                        operator = "===";
                    }
                    
                    operators = {
                        '==': function (l, r) { return l == r; },
                        '===': function (l, r) { return l === r; },
                        '!=': function (l, r) { return l != r; },
                        '!==': function (l, r) { return l !== r; },
                        '<': function (l, r) { return l < r; },
                        '>': function (l, r) { return l > r; },
                        '<=': function (l, r) { return l <= r; },
                        '>=': function (l, r) { return l >= r; },
                        'typeof': function (l, r) { return typeof l == r; }
                    };
                    
                    if (!operators[operator]) {
                        throw new Error("Handlerbars Helper 'compare' doesn't know the operator " + operator);
                    }
                    
                    result = operators[operator](lvalue, rvalue);
                    
                    if (result) {
                        return options.fn(this);
                    } else {
                        return options.inverse(this);
                    }

                });


                // init view templates
                app.initView('alert');
                app.initView('menu');
                app.initView('share');
                app.initView('profile');
                app.initView('pitcher');
                app.initView('detail');
                app.initView('burolist');
                app.initView('invite');
                
                app.initView('thnx');
                app.initView('help');
                app.initView('about');
                app.initView('selectcount');


                // init router
                router.add("/thnx", function() {
                    app.gotoPage('thnx');
                });
                router.add("/about", function() {
                    app.gotoPage('closemenu');
                    app.gotoPage('about');

                });
                router.add("/help", function() {
                    app.gotoPage('closemenu');
                    app.gotoPage('help');
                });
                router.add("/profile", function() {
                    if( app.model.searchStarted ) {
                        app.alertview.show('warn');
                        return;
                    }

                    app.initView('profile', app.model.typesJSON );
                    app.gotoPage('closemenu');
                    app.gotoPage('profile');
                });
                router.add("/pitcher", function() {

                  
                    app.initView('pitcher', app.model.getSortedAgencies());
                    app.gotoPage('pitcher');
                });
                router.add("/detail", function() {

                    app.initView('detail', app.model.detailAgency );
                    app.gotoPage('detail');
                });
                router.add("/burolist", function() {
                    app.initView('burolist', app.model.selectedAgencies );
                    app.gotoPage('burolist');
                });
                router.add("/invite", function() {
                    app.initView( 'invite', app.model );
                    app.gotoPage('invite');
                });
                router.add("/menu", function() {

                    app.gotoPage('menu');
                });
                router.add("/closemenu", function() {
                    app.gotoPage('closemenu');
                });
                router.start();

                break;
            case 'menubutton':
                router.goto('/menu');
                break;
            case 'backbutton':
                break;
            case 'online':
                break;
            case 'offline':
                break;
            case 'pause':
                break;
            case 'resume':
                break;
        }
    },
    openShare: function() {
        app.shareview.show();
        
    },
    closeShare: function() {
        app.shareview.hide();
        
    },
    share :function  (type) {
       
    },
    hideKeyboard : function() {
       
    },
    // init views
    initView: function(viewname, context) {

        var source;
        var template;
        var viewname = viewname
        if (!context) context = {};
        var context = context;
        var html;
        // check if template already loaded
        if (app.templates[viewname]) {
         
            // compile template
            template = app.templates[viewname];
            html = template(context);
            $("#" + viewname + "view").html(html);
            return;
        }
        // LOAD handlebars template file
        $.ajax({
            url: 'templates/' + viewname + ".hbs",
            success: function(data) {
                source = data;
                template = Handlebars.compile(source);

                // compile template
                html = template(context);
               $("#" + viewname + "view").html(html);
                
                // store template
                app.templates[viewname] = template;
            
                // call init function
                app[viewname + 'view'].init();
                if( viewname == 'selectcount') {

                    app.model.loadJSON();

                }
            }
        })
    },
    assetLoaded : function  () {
        
        if( app.model.agenciesJSON && app.model.typesJSON ) {
            


            router.goto('/profile');
            router.goto('/menu');

            app.model.welcomeMsg = window.localStorage.getItem("welcomeMsg");
       
            if( !app.model.welcomeMsg ) {
                app.alertview.show('welcome');
                window.localStorage.setItem("welcomeMsg", true);
            }

            navigator.splashscreen.hide();

            app.model.preloadImages();

        } 
    }
};